
/**********************************************************************/
/*	JDK-1.0 / JNI (JDK-1.1) Native Interface	*/

#define MK_STRING(x)	#x	/* ANSI C preprocessor	*/

extern ClassClass	*MPIconstClass;

/**********************************************************************/
#ifdef MPI_USE_JNI		/* JNI (JDK-1.1)	*/

#include <jni.h>

#define JNIENV		JNIEnv *env,
#define JOBJECT(typ)	jobject
#define JARRAY(typ)	jarray
#define JSTRING		jstring

#define Jint		jint
#define Jvoid		jvoid

/*	Hidden references to: env			*/
/*	`field' or `name' is an unquoted field name	*/

#define DECL_FIELD_ID(obj, field)	jfieldID CAT(obj, field)

#define GET_FIELD_ID(obj, name, sig)	\
			CAT(obj,name) = (*env)->GetFieldID(env, GetObjectClass(env, obj), MK_STRING(name), sig)
#define GET_INT_FIELD(obj, field, typ, var)	\
			var = typ GetIntField(env, obj, CAT(obj,field))
#define SET_INT_FIELD(obj, field, value)	\
			(*env)->SetIntField(env, obj, CAT(obj,field), value)
#define GET_OBJECT_FIELD(obj, field, typ, var)	\
			var = typ GetObjectField(env, obj, CAT(obj,field))
#define SET_OBJECT_FIELD(obj, field, value)	\
			(*env)->SetObjectField(env, obj, CAT(obj,field), value)
#define GET_OBJECT_ARRAY_ELEMENT(array, index)	\
			(*env)->GetObjectArrayElement(env, array, index)
#define SET_OBJECT_ARRAY_ELEMENT(array, index, value)	\
			(*env)->SetObjectArrayElement(env, array, index, value)

#define GET_ARRAY_LENGTH(array)	\
			(*env)->GetArrayLength(env, array)

/*		JDK-1.1 functions:
GetFieldID(JNIEnv *env, jclass clazz, const char *name, const char *sig);

jclass GetObjectClass(JNIEnv *env, jobject obj);

NativeType GetObjectField(JNIEnv *env, jobject obj, jfieldID fieldID);

void SetObjectField(JNIEnv *env, jobject obj, jfieldID fieldID,
			NativeType value);

jobject GetObjectArrayElement(JNIEnv *env, jarray array, jsize index);

void SetObjectArrayElement(JNIEnv *env, jarray array, 
			jsize index, jobject value);
*/

/**********************************************************************/
#else			/* JDK-1.0		*/

#define JNIENV
#define JOBJECT(typ)	struct typ *
#define JARRAY(typ)	typ *
#define JSTRING		HString *

#define Jint		long
#define Jvoid		void

#define Jchar		long
#define Jshort		long
#define Jlong		int64_t
#define Jfloat		float
#define Jdouble		double_t

/*
#define Jint		int
#define Jvoid		void

#define Jchar		char
#define Jshort		short
#define Jlong		long
#define Jfloat		float
#define Jdouble		double
*/

#define DECL_FIELD_ID(obj, field)

#define GET_FIELD_ID(obj, name, sig)
#define GET_INT_FIELD(obj, field, typ, var)	\
			var = typ unhand(obj)->field
#define SET_INT_FIELD(obj, field, value)	\
			unhand(obj)->field = value
#define GET_OBJECT_FIELD(obj, field, typ, var)	\
			var = typ unhand(obj)->field
#define SET_OBJECT_FIELD(obj, field, value)	\
			unhand(obj)->field = value
#define GET_OBJECT_ARRAY_ELEMENT(array, index)	\
			unhand(array)->body[index]
#define SET_OBJECT_ARRAY_ELEMENT(array, index, value)	\
			unhand(array)->body[index] = value

#define GET_ARRAY_FIELD(obj, field, typ, var)	\
			var = typ unhand(obj)->field
#define SET_ARRAY_FIELD(obj, field, value)	\
			unhand(obj)->field = value

#define GET_ARRAY_ARRAY_ELEMENT(array, index)	\
			unhand(array)->body[index]
#define SET_ARRAY_ARRAY_ELEMENT(array, index, value)	\
			unhand(array)->body[index] = value

#define GET_CHAR_FIELD(obj, field, typ, var)	\
			var = typ unhand(obj)->field
#define GET_SHORT_FIELD(obj, field, typ, var)	\
			var = typ unhand(obj)->field
#define GET_LONG_FIELD(obj, field, typ, var)	\
	memcpy (&var, &unhand(obj)->field, sizeof (Jlong));
/*			var = typ unhand(obj)->field */
#define GET_FLOAT_FIELD(obj, field, typ, var)	\
			var = typ unhand(obj)->field
#define GET_DOUBLE_FIELD(obj, field, typ, var)	\
			var = typ unhand(obj)->field

#define SET_CHAR_FIELD(obj, field, value)	\
			unhand(obj)->field = value
#define SET_SHORT_FIELD(obj, field, value)	\
			unhand(obj)->field = value
#define SET_LONG_FIELD(obj, field, value)	\
			unhand(obj)->field = value
#define SET_FLOAT_FIELD(obj, field, value)	\
			unhand(obj)->field = value
#define SET_DOUBLE_FIELD(obj, field, value)	\
			unhand(obj)->field = value

#define GET_STRING_CHARS(jstr) \
			makeCString (jstr)
#define NEW_STRING(cstr, len) \
			(JSTRING)makeJavaString (cstr, len)

/*	This uses java/lang/reflect/Array from JDK1.1
#define GET_ARRAY_LENGTH(array)	\
	execute_java_static_method(0,	\
		FindClass(0, "java/lang/reflect/Array", 1),	\
		"getLength", "(Ljava/lang/Object;)I", array)
*/
#define GET_ARRAY_LENGTH(array)	\
	execute_java_static_method(0,	\
		MPIconstClass,	\
		"getLength", "([Ljava/lang/Object;)I", array)
#define GET_INT_ARRAY_ELEMENTS(array)	\
			unhand(array)->body
#define GET_CHAR_ARRAY_ELEMENTS(array)	\
			unhand(array)->body
#define GET_SHORT_ARRAY_ELEMENTS(array)	\
			unhand(array)->body
#define GET_LONG_ARRAY_ELEMENTS(array)	\
			unhand(array)->body
#define GET_FLOAT_ARRAY_ELEMENTS(array)	\
			unhand(array)->body
#define GET_DOUBLE_ARRAY_ELEMENTS(array)	\
			unhand(array)->body

#define RELEASE_INT_ARRAY_ELEMENTS(array, elems)
#define RELEASE_CHAR_ARRAY_ELEMENTS(array, elems)
#define RELEASE_SHORT_ARRAY_ELEMENTS(array, elems)
#define RELEASE_LONG_ARRAY_ELEMENTS(array, elems)
#define RELEASE_FLOAT_ARRAY_ELEMENTS(array, elems)
#define RELEASE_DOUBLE_ARRAY_ELEMENTS(array, elems)

#endif

/**********************************************************************/
/*	Machine-dependent definitions		*/

#define C2J_CONV_char(x)	x
#define C2J_CONV_int(x)		x
#define C2J_CONV_long(x)	int2ll(x)
#define C2J_CONV_short(x)	x
#define C2J_CONV_float(x)	x
#define C2J_CONV_double(x)	x

#define J2C_CONV_char(x)	x
#define J2C_CONV_int(x)		x
#define J2C_CONV_long(x)	ll2int(x)
#define J2C_CONV_short(x)	x
#define J2C_CONV_float(x)	x
#define J2C_CONV_double(x)	x

/**********************************************************************/


